package sun.print;

import java.util.Map;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.Paint;
import java.awt.Composite;
import java.awt.font.GlyphVector;
import java.awt.image.BufferedImageOp;
import java.awt.image.BufferedImage;
import java.awt.image.renderable.RenderableImage;
import java.awt.geom.AffineTransform;
import java.awt.image.RenderedImage;
import java.awt.Image;
import java.awt.font.TextLayout;
import java.text.AttributedCharacterIterator;
import java.awt.geom.RoundRectangle2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Line2D;
import java.awt.Shape;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import sun.java2d.Spans;
import java.awt.print.PrinterJob;
import java.awt.image.ImageObserver;
import java.awt.print.PrinterGraphics;
import java.awt.Graphics2D;

public class PeekGraphics extends Graphics2D implements PrinterGraphics, ImageObserver, Cloneable
{
    Graphics2D mGraphics;
    PrinterJob mPrinterJob;
    private Spans mDrawingArea;
    private PeekMetrics mPrintMetrics;
    private boolean mAWTDrawingOnly;
    
    public PeekGraphics(Graphics2D mGraphics, PrinterJob mPrinterJob) {
        this.mDrawingArea = new Spans();
        this.mPrintMetrics = new PeekMetrics();
        this.mAWTDrawingOnly = false;
        this.mGraphics = mGraphics;
        this.mPrinterJob = mPrinterJob;
    }
    
    public Graphics2D getDelegate() {
        return this.mGraphics;
    }
    
    public void setDelegate(Graphics2D mGraphics) {
        this.mGraphics = mGraphics;
    }
    
    @Override
    public PrinterJob getPrinterJob() {
        return this.mPrinterJob;
    }
    
    public void setAWTDrawingOnly() {
        this.mAWTDrawingOnly = true;
    }
    
    public boolean getAWTDrawingOnly() {
        return this.mAWTDrawingOnly;
    }
    
    public Spans getDrawingArea() {
        return this.mDrawingArea;
    }
    
    @Override
    public GraphicsConfiguration getDeviceConfiguration() {
        return ((RasterPrinterJob)this.mPrinterJob).getPrinterGraphicsConfig();
    }
    
    @Override
    public Graphics create() {
        PeekGraphics peekGraphics = null;
        try {
            peekGraphics = (PeekGraphics)this.clone();
            peekGraphics.mGraphics = (Graphics2D)this.mGraphics.create();
        }
        catch (CloneNotSupportedException ex) {}
        return peekGraphics;
    }
    
    @Override
    public void translate(int n, int n2) {
        this.mGraphics.translate(n, n2);
    }
    
    @Override
    public void translate(double n, double n2) {
        this.mGraphics.translate(n, n2);
    }
    
    @Override
    public void rotate(double n) {
        this.mGraphics.rotate(n);
    }
    
    @Override
    public void rotate(double n, double n2, double n3) {
        this.mGraphics.rotate(n, n2, n3);
    }
    
    @Override
    public void scale(double n, double n2) {
        this.mGraphics.scale(n, n2);
    }
    
    @Override
    public void shear(double n, double n2) {
        this.mGraphics.shear(n, n2);
    }
    
    @Override
    public Color getColor() {
        return this.mGraphics.getColor();
    }
    
    @Override
    public void setColor(Color color) {
        this.mGraphics.setColor(color);
    }
    
    @Override
    public void setPaintMode() {
        this.mGraphics.setPaintMode();
    }
    
    @Override
    public void setXORMode(Color xorMode) {
        this.mGraphics.setXORMode(xorMode);
    }
    
    @Override
    public Font getFont() {
        return this.mGraphics.getFont();
    }
    
    @Override
    public void setFont(Font font) {
        this.mGraphics.setFont(font);
    }
    
    @Override
    public FontMetrics getFontMetrics(Font font) {
        return this.mGraphics.getFontMetrics(font);
    }
    
    @Override
    public FontRenderContext getFontRenderContext() {
        return this.mGraphics.getFontRenderContext();
    }
    
    @Override
    public Rectangle getClipBounds() {
        return this.mGraphics.getClipBounds();
    }
    
    @Override
    public void clipRect(int n, int n2, int n3, int n4) {
        this.mGraphics.clipRect(n, n2, n3, n4);
    }
    
    @Override
    public void setClip(int n, int n2, int n3, int n4) {
        this.mGraphics.setClip(n, n2, n3, n4);
    }
    
    @Override
    public Shape getClip() {
        return this.mGraphics.getClip();
    }
    
    @Override
    public void setClip(Shape clip) {
        this.mGraphics.setClip(clip);
    }
    
    @Override
    public void copyArea(int n, int n2, int n3, int n4, int n5, int n6) {
    }
    
    @Override
    public void drawLine(int n, int n2, int n3, int n4) {
        this.addStrokeShape(new Line2D.Float((float)n, (float)n2, (float)n3, (float)n4));
        this.mPrintMetrics.draw(this);
    }
    
    @Override
    public void fillRect(int n, int n2, int n3, int n4) {
        this.addDrawingRect(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
        this.mPrintMetrics.fill(this);
    }
    
    @Override
    public void clearRect(int n, int n2, int n3, int n4) {
        this.addDrawingRect(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
        this.mPrintMetrics.clear(this);
    }
    
    @Override
    public void drawRoundRect(int n, int n2, int n3, int n4, int n5, int n6) {
        this.addStrokeShape(new RoundRectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6));
        this.mPrintMetrics.draw(this);
    }
    
    @Override
    public void fillRoundRect(int n, int n2, int n3, int n4, int n5, int n6) {
        this.addDrawingRect(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
        this.mPrintMetrics.fill(this);
    }
    
    @Override
    public void drawOval(int n, int n2, int n3, int n4) {
        this.addStrokeShape(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
        this.mPrintMetrics.draw(this);
    }
    
    @Override
    public void fillOval(int n, int n2, int n3, int n4) {
        this.addDrawingRect(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
        this.mPrintMetrics.fill(this);
    }
    
    @Override
    public void drawArc(int n, int n2, int n3, int n4, int n5, int n6) {
        this.addStrokeShape(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
        this.mPrintMetrics.draw(this);
    }
    
    @Override
    public void fillArc(int n, int n2, int n3, int n4, int n5, int n6) {
        this.addDrawingRect(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
        this.mPrintMetrics.fill(this);
    }
    
    @Override
    public void drawPolyline(int[] array, int[] array2, int n) {
        if (n > 0) {
            int n2 = array[0];
            int n3 = array2[0];
            for (int i = 1; i < n; n2 = array[i], n3 = array2[i], ++i) {
                this.drawLine(n2, n3, array[i], array2[i]);
            }
        }
    }
    
    @Override
    public void drawPolygon(int[] array, int[] array2, int n) {
        if (n > 0) {
            this.drawPolyline(array, array2, n);
            this.drawLine(array[n - 1], array2[n - 1], array[0], array2[0]);
        }
    }
    
    @Override
    public void fillPolygon(int[] array, int[] array2, int n) {
        if (n > 0) {
            int n2 = array[0];
            int n3 = array2[0];
            int n4 = array[0];
            int n5 = array2[0];
            for (int i = 1; i < n; ++i) {
                if (array[i] < n2) {
                    n2 = array[i];
                }
                else if (array[i] > n4) {
                    n4 = array[i];
                }
                if (array2[i] < n3) {
                    n3 = array2[i];
                }
                else if (array2[i] > n5) {
                    n5 = array2[i];
                }
            }
            this.addDrawingRect((float)n2, (float)n3, (float)(n4 - n2), (float)(n5 - n3));
        }
        this.mPrintMetrics.fill(this);
    }
    
    @Override
    public void drawString(String s, int n, int n2) {
        this.drawString(s, (float)n, (float)n2);
    }
    
    @Override
    public void drawString(AttributedCharacterIterator attributedCharacterIterator, int n, int n2) {
        this.drawString(attributedCharacterIterator, (float)n, (float)n2);
    }
    
    @Override
    public void drawString(AttributedCharacterIterator text, float x, float y) {
        if (text == null) {
            throw new NullPointerException("AttributedCharacterIterator is null");
        }
        new TextLayout(text, this.getFontRenderContext()).draw(this, x, y);
        return;
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        PeekGraphics.ImageWaiter imageWaiter = new PeekGraphics.ImageWaiter(image);
        this.addDrawingRect((float)n, (float)n2, (float)imageWaiter.getWidth(), (float)imageWaiter.getHeight());
        this.mPrintMetrics.drawImage(this, image);
        return this.mGraphics.drawImage(image, n, n2, imageObserver);
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, int n3, int n4, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        this.addDrawingRect((float)n, (float)n2, (float)n3, (float)n4);
        this.mPrintMetrics.drawImage(this, image);
        return this.mGraphics.drawImage(image, n, n2, n3, n4, imageObserver);
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, Color color, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        PeekGraphics.ImageWaiter imageWaiter = new PeekGraphics.ImageWaiter(image);
        this.addDrawingRect((float)n, (float)n2, (float)imageWaiter.getWidth(), (float)imageWaiter.getHeight());
        this.mPrintMetrics.drawImage(this, image);
        return this.mGraphics.drawImage(image, n, n2, color, imageObserver);
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, int n3, int n4, Color color, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        this.addDrawingRect((float)n, (float)n2, (float)n3, (float)n4);
        this.mPrintMetrics.drawImage(this, image);
        return this.mGraphics.drawImage(image, n, n2, n3, n4, color, imageObserver);
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        this.addDrawingRect((float)n, (float)n2, (float)(n3 - n), (float)(n4 - n2));
        this.mPrintMetrics.drawImage(this, image);
        return this.mGraphics.drawImage(image, n, n2, n3, n4, n5, n6, n7, n8, imageObserver);
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, Color color, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        this.addDrawingRect((float)n, (float)n2, (float)(n3 - n), (float)(n4 - n2));
        this.mPrintMetrics.drawImage(this, image);
        return this.mGraphics.drawImage(image, n, n2, n3, n4, n5, n6, n7, n8, color, imageObserver);
    }
    
    @Override
    public void drawRenderedImage(RenderedImage renderedImage, AffineTransform affineTransform) {
        if (renderedImage == null) {
            return;
        }
        this.mPrintMetrics.drawImage(this, renderedImage);
        this.mDrawingArea.addInfinite();
        return;
    }
    
    @Override
    public void drawRenderableImage(RenderableImage renderableImage, AffineTransform affineTransform) {
        if (renderableImage == null) {
            return;
        }
        this.mPrintMetrics.drawImage(this, renderableImage);
        this.mDrawingArea.addInfinite();
        return;
    }
    
    @Override
    public void dispose() {
        this.mGraphics.dispose();
    }
    
    @Override
    public void finalize() {
    }
    
    @Override
    public void draw(Shape shape) {
        this.addStrokeShape(shape);
        this.mPrintMetrics.draw(this);
    }
    
    @Override
    public boolean drawImage(Image image, AffineTransform affineTransform, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        this.mDrawingArea.addInfinite();
        this.mPrintMetrics.drawImage(this, image);
        return this.mGraphics.drawImage(image, affineTransform, imageObserver);
    }
    
    @Override
    public void drawImage(BufferedImage bufferedImage, BufferedImageOp bufferedImageOp, int n, int n2) {
        if (bufferedImage == null) {
            return;
        }
        this.mPrintMetrics.drawImage(this, bufferedImage);
        this.mDrawingArea.addInfinite();
        return;
    }
    
    @Override
    public void drawString(String str, float n, float n2) {
        if (str.length() == 0) {
            return;
        }
        this.addDrawingRect(this.getFont().getStringBounds(str, this.getFontRenderContext()), n, n2);
        this.mPrintMetrics.drawText(this);
        return;
    }
    
    @Override
    public void drawGlyphVector(GlyphVector glyphVector, float n, float n2) {
        this.addDrawingRect(glyphVector.getLogicalBounds(), n, n2);
        this.mPrintMetrics.drawText(this);
    }
    
    @Override
    public void fill(Shape shape) {
        this.addDrawingRect(shape.getBounds());
        this.mPrintMetrics.fill(this);
    }
    
    @Override
    public boolean hit(Rectangle rectangle, Shape shape, boolean b) {
        return this.mGraphics.hit(rectangle, shape, b);
    }
    
    @Override
    public void setComposite(Composite composite) {
        this.mGraphics.setComposite(composite);
    }
    
    @Override
    public void setPaint(Paint paint) {
        this.mGraphics.setPaint(paint);
    }
    
    @Override
    public void setStroke(Stroke stroke) {
        this.mGraphics.setStroke(stroke);
    }
    
    @Override
    public void setRenderingHint(RenderingHints.Key key, Object o) {
        this.mGraphics.setRenderingHint(key, o);
    }
    
    @Override
    public Object getRenderingHint(RenderingHints.Key key) {
        return this.mGraphics.getRenderingHint(key);
    }
    
    @Override
    public void setRenderingHints(Map<?, ?> renderingHints) {
        this.mGraphics.setRenderingHints(renderingHints);
    }
    
    @Override
    public void addRenderingHints(Map<?, ?> map) {
        this.mGraphics.addRenderingHints(map);
    }
    
    @Override
    public RenderingHints getRenderingHints() {
        return this.mGraphics.getRenderingHints();
    }
    
    @Override
    public void transform(AffineTransform affineTransform) {
        this.mGraphics.transform(affineTransform);
    }
    
    @Override
    public void setTransform(AffineTransform transform) {
        this.mGraphics.setTransform(transform);
    }
    
    @Override
    public AffineTransform getTransform() {
        return this.mGraphics.getTransform();
    }
    
    @Override
    public Paint getPaint() {
        return this.mGraphics.getPaint();
    }
    
    @Override
    public Composite getComposite() {
        return this.mGraphics.getComposite();
    }
    
    @Override
    public void setBackground(Color background) {
        this.mGraphics.setBackground(background);
    }
    
    @Override
    public Color getBackground() {
        return this.mGraphics.getBackground();
    }
    
    @Override
    public Stroke getStroke() {
        return this.mGraphics.getStroke();
    }
    
    @Override
    public void clip(Shape shape) {
        this.mGraphics.clip(shape);
    }
    
    public boolean hitsDrawingArea(Rectangle rectangle) {
        return this.mDrawingArea.intersects((float)rectangle.getMinY(), (float)rectangle.getMaxY());
    }
    
    public PeekMetrics getMetrics() {
        return this.mPrintMetrics;
    }
    
    private void addDrawingRect(Rectangle2D rectangle2D, float n, float n2) {
        this.addDrawingRect((float)(rectangle2D.getX() + (double)n), (float)(rectangle2D.getY() + (double)n2), (float)rectangle2D.getWidth(), (float)rectangle2D.getHeight());
    }
    
    private void addDrawingRect(float x, float y, float w, float h) {
        this.addDrawingRect(new Rectangle2D.Float(x, y, w, h));
    }
    
    private void addDrawingRect(Rectangle2D pSrc) {
        Rectangle2D bounds2D = this.getTransform().createTransformedShape(pSrc).getBounds2D();
        this.mDrawingArea.add((float)bounds2D.getMinY(), (float)bounds2D.getMaxY());
    }
    
    private void addStrokeShape(Shape shape) {
        this.addDrawingRect(this.getStroke().createStrokedShape(shape).getBounds2D());
    }
    
    @Override
    public synchronized boolean imageUpdate(Image image, int n, int n2, int n3, int n4, int n5) {
        boolean n6 = false;
        if ((n & 0x3) != 0x0) {
            n6 = true;
            this.notify();
        }
        return n6;
    }
    
    private synchronized int getImageWidth(Image image) {
        while (image.getWidth(this) == -1) {
            try {
                this.wait();
            }
            catch (InterruptedException ex) {}
        }
        return image.getWidth(this);
    }
    
    private synchronized int getImageHeight(Image image) {
        while (image.getHeight(this) == -1) {
            try {
                this.wait();
            }
            catch (InterruptedException ex) {}
        }
        return image.getHeight(this);
    }
    
    protected class ImageWaiter implements ImageObserver
    {
        private int mWidth;
        private int mHeight;
        private boolean badImage;
        
        ImageWaiter(Image image) {
            this.badImage = false;
            this.waitForDimensions(image);
        }
        
        public int getWidth() {
            return this.mWidth;
        }
        
        public int getHeight() {
            return this.mHeight;
        }
        
        private synchronized void waitForDimensions(Image image) {
            this.mHeight = image.getHeight(this);
            this.mWidth = image.getWidth(this);
            while (!this.badImage) {
                if (this.mWidth >= 0 && this.mHeight >= 0) {
                    break;
                }
                try {
                    Thread.sleep(50L);
                }
                catch (InterruptedException ex) {}
                this.mHeight = image.getHeight(this);
                this.mWidth = image.getWidth(this);
            }
            if (this.badImage) {
                this.mHeight = 0;
                this.mWidth = 0;
            }
        }
        
        @Override
        public synchronized boolean imageUpdate(Image image, int n, int n2, int n3, int n4, int n5) {
            int n6 = (n & 0xC2) != 0x0;
            this.badImage = ((n & 0xC0) != 0x0);
            return n6;
        }
    }
}
